@@ -34,6 +34,12 @@
 #include "safetime.h"
 #if defined(_WIN32)
 #include <io.h>
+#ifndef S_IXUSR
+#define S_IXUSR    0100
+#endif
+#ifndef S_IWUSR
+#define S_IWUSR    0200
+#endif
 #else
 #include <unistd.h>
 #endif
@@ -74,10 +80,16 @@
  *
  * File_Exists --
  *
- *      Check if a file exists.
+ *      Check if a file is accessible with the process' real user ID
+ *
+ *      XXX - This function invokes access(), which uses the real uid,
+ *      not the effective uid, so it probably does not do what you 
+ *      expect.  Instead it should use FileAttributes(), which calls
+ *      Posix_Stat() and uses the effective uid, but it's too risky 
+ *      to fix right now.  See PR 459242.
  *
  * Results:
- *      TRUE    file exists
+ *      TRUE    file is accessible with the process' real uid
  *      FALSE   file doesn't exist or an error occured
  *      
  * Side effects:
@@ -87,7 +99,7 @@
  */
 
 Bool
-File_Exists(ConstUnicode pathName)  // IN:
+File_Exists(ConstUnicode pathName)  // IN: May be NULL.
 {
    return FileIO_IsSuccess(FileIO_Access(pathName, FILEIO_ACCESS_EXISTS));
 }
@@ -152,6 +164,47 @@ File_IsDirectory(ConstUnicode pathName)  // IN:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * File_GetFilePermissions --
+ *
+ *	Return the read / write / execute permissions of a file.
+ *
+ * Results:
+ *	TRUE if success, FALSE otherwise.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+File_GetFilePermissions(ConstUnicode pathName,  // IN:
+                        int *mode)              // OUT: file mode
+{
+   FileData fileData;
+
+   ASSERT(mode);
+   if (FileAttributes(pathName, &fileData) != 0) {
+      return FALSE;
+   }
+   *mode = fileData.fileMode;
+#if defined(_WIN32)
+      /*
+       *  On Win32 implementation of FileAttributes does not return execution
+       * bit.
+       */
+
+      if (FileIO_Access(pathName, FILEIO_ACCESS_EXEC) == FILEIO_SUCCESS) {
+         *mode |= S_IXUSR;
+      }
+#endif
+   return TRUE;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -204,8 +257,6 @@ File_UnlinkNoFollow(ConstUnicode pathName)  // IN:
 }
 
 
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -319,10 +370,11 @@ File_DeleteEmptyDirectory(ConstUnicode pathName)  // IN:
  *      Return the old machineID, the one based on Hostinfo_MachineID.
  *
  * Results:
- *      The machineID is returned.
+ *      The machineID is returned. It should not be freed.
  *
  * Side effects:
- *      None.
+ *      Memory allocated for the machineID is never freed, however the
+ *      memory is cached - there is no memory leak.
  *
  *----------------------------------------------------------------------
  */
@@ -386,7 +438,7 @@ GetOldMachineID(void)
  *      fails drop back to the older machineID method.
  *
  * Results:
- *      The machineID is returned.
+ *      The machineID is returned. It should not be freed.
  *
  * Side effects:
  *      Memory allocated for the machineID is never freed, however the
@@ -421,7 +473,11 @@ FileLockGetMachineID(void)
        */
 
 #if defined(_WIN32)
+#  if !defined(__MINGW32__)
       q = UUID_GetRealHostUUID();
+#  else
+      q = NULL;
+#  endif
 #elif defined(__APPLE__) || defined(VMX86_SERVER)
       q = UUID_GetHostUUID();
 #else
@@ -429,7 +485,7 @@ FileLockGetMachineID(void)
 #endif
 
       if (q == NULL) {
-         p = (char *) GetOldMachineID();
+         p = Util_SafeStrdup(GetOldMachineID());
       } else {
          p = Str_SafeAsprintf(NULL, "uuid=%s", q);
          free(q);
@@ -855,6 +911,45 @@ File_SplitName(ConstUnicode pathName,  // IN:
 }
 
 
+/*
+ *---------------------------------------------------------------------------
+ *
+ * File_PathJoin --
+ *
+ *      Join the dirName and baseName together to create a (full) path but
+ *      don't add a DIRSEPS unless necessary.
+ *
+ *      File_PathJoin("a", "b")  -> "a/b"
+ *      File_PathJoin("a/", "b") -> "a/b"
+ *
+ * Results: 
+ *      The constructed path which must be freed by the caller.
+ *
+ * Side effects: 
+ *      None
+ *
+ *---------------------------------------------------------------------------
+ */
+
+Unicode
+File_PathJoin(ConstUnicode dirName,   // IN:
+              ConstUnicode baseName)  // IN:
+{
+   Unicode result;
+
+   ASSERT(dirName);
+   ASSERT(baseName);
+
+   if (Unicode_EndsWith(dirName, DIRSEPS)) {
+      result = Unicode_Append(dirName, baseName);
+   } else {
+      result = Unicode_Join(dirName, DIRSEPS, baseName, NULL);
+   }
+
+   return result;
+}
+
+
 /*
  *---------------------------------------------------------------------------
  *
@@ -970,7 +1065,8 @@ File_StripSlashes(ConstUnicode path) // IN
        * Don't strip first slash on Windows, since we want at least
        * one slash to trail a drive letter/colon or UNC specifier.
        */
-#ifdef _WIN32
+
+#if defined(_WIN32)
       while ((i > 1) && (('/' == dir2[i - 1]) ||
                          ('\\' == dir2[i - 1]))) {
 #else
@@ -1023,6 +1119,7 @@ File_MakeTempEx(ConstUnicode dir,       // IN:
                 ConstUnicode fileName,  // IN:
                 Unicode *presult)       // OUT:
 {
+#ifndef _WIN32
    int fd;
    int err;
    uint32 var;
@@ -1086,6 +1183,10 @@ File_MakeTempEx(ConstUnicode dir,       // IN:
    errno = err;
 
    return fd;
+#else
+   // XXX - need a Windows/mingw version of this function.
+   return -1;
+#endif
 }
 
 
@@ -1180,7 +1281,7 @@ File_CopyFromFdToFd(FileIODescriptor src,  // IN:
    FileIOResult fretR;
 
    do {
-      unsigned char buf[1024];
+      unsigned char buf[8 * 1024];
       size_t actual;
       FileIOResult fretW;
 
@@ -1188,6 +1289,7 @@ File_CopyFromFdToFd(FileIODescriptor src,  // IN:
       if (!FileIO_IsSuccess(fretR) && (fretR != FILEIO_READ_ERROR_EOF)) {
          Msg_Append(MSGID(File.CopyFromFdToFd.read.failure)
                                "Read error: %s.\n\n", FileIO_MsgError(fretR));
+
          return FALSE;
       }
 
@@ -1195,6 +1297,7 @@ File_CopyFromFdToFd(FileIODescriptor src,  // IN:
       if (!FileIO_IsSuccess(fretW)) {
          Msg_Append(MSGID(File.CopyFromFdToFd.write.failure)
                               "Write error: %s.\n\n", FileIO_MsgError(fretW));
+
          return FALSE;
       }
    } while (fretR != FILEIO_READ_ERROR_EOF);
@@ -1259,6 +1362,11 @@ File_CopyFromFdToName(FileIODescriptor src,  // IN:
       result = FALSE;
    }
 
+   if (result == FALSE) {
+      /* The copy failed: ensure the destination file is removed */
+      File_Unlink(dstName);
+   }
+
    return result;
 }
 
@@ -1371,6 +1479,7 @@ File_CopyFromNameToName(ConstUnicode srcName,  // IN:
       Msg_Append(MSGID(File.CopyFromNameToName.open.failure)
                  "Unable to open the '%s' file for read access: %s.\n\n",
                  UTF8(srcName), FileIO_MsgError(fret));
+
       return FALSE;
    }
 
@@ -1427,6 +1536,7 @@ File_CopyFromFd(FileIODescriptor src,     // IN:
       Msg_Append(MSGID(File.CopyFromFdToName.create.failure)
                  "Unable to create a new '%s' file: %s.\n\n", UTF8(dstName),
                  FileIO_MsgError(fret));
+
       return FALSE;
    }
 
@@ -1439,6 +1549,11 @@ File_CopyFromFd(FileIODescriptor src,     // IN:
       result = FALSE;
    }
 
+   if (result == FALSE) {
+      /* The copy failed: ensure the destination file is removed */
+      File_Unlink(dstName);
+   }
+
    return result;
 }
 
@@ -1482,6 +1597,7 @@ File_Copy(ConstUnicode srcName,    // IN:
       Msg_Append(MSGID(File.Copy.open.failure)
                  "Unable to open the '%s' file for read access: %s.\n\n",
                  UTF8(srcName), FileIO_MsgError(fret));
+
       return FALSE;
    }
 
@@ -1630,6 +1746,81 @@ File_SupportsLargeFiles(ConstUnicode pathName)  // IN:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * File_MapPathPrefix --
+ *
+ *      Given a path and a newPrefix -> oldPrefix mapping, transform
+ *      oldPath according to the mapping.
+ *
+ * Results:
+ *      The new path, or NULL if there is no mapping.
+ *
+ * Side effects:
+ *      The returned string is allocated, free it.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+char *
+File_MapPathPrefix(const char *oldPath,               // IN
+                   const char **oldPrefixes,          // IN
+                   const char **newPrefixes,          // IN
+                   size_t numPrefixes)                // IN
+{
+   int i;
+   size_t oldPathLen = strlen(oldPath);
+
+   for (i = 0; i < numPrefixes; i++) {
+      char *newPath;
+      char *oldPrefix;
+      char *newPrefix;
+      size_t oldPrefixLen;
+
+      oldPrefix = File_StripSlashes(oldPrefixes[i]);
+      newPrefix = File_StripSlashes(newPrefixes[i]);
+      oldPrefixLen = strlen(oldPrefix);
+
+      /*
+       * If the prefix matches on a DIRSEPS boundary, or the prefix is the
+       * whole string, replace it.
+       * If we don't insist on matching a whole directory name, we could
+       * mess things of if one directory is a substring of another.
+       */
+
+      if ((oldPathLen >= oldPrefixLen) &&
+          (memcmp(oldPath, oldPrefix, oldPrefixLen) == 0) &&
+          (strchr(VALID_DIRSEPS, oldPath[oldPrefixLen]) ||
+              (oldPath[oldPrefixLen] == '\0'))) {
+         size_t newPrefixLen = strlen(newPrefix);
+         size_t newPathLen = (oldPathLen - oldPrefixLen) + newPrefixLen;
+
+         ASSERT(newPathLen > 0);
+         ASSERT(oldPathLen >= oldPrefixLen);
+
+         newPath = Util_SafeMalloc((newPathLen + 1) * sizeof(char));
+         memcpy(newPath, newPrefix, newPrefixLen);
+         memcpy(newPath + newPrefixLen, oldPath + oldPrefixLen,
+                oldPathLen - oldPrefixLen + 1);
+         /*
+          * It should only match once.  Weird self-referencing mappings
+          * aren't allowed.
+          */
+
+         free(oldPrefix);
+         free(newPrefix);
+
+         return newPath;
+      }
+      free(oldPrefix);
+      free(newPrefix);
+   }
+
+   return NULL;
+}
+
+
 #if !defined(N_PLAT_NLM)
 /*
  *----------------------------------------------------------------------------
@@ -1736,11 +1927,12 @@ File_CreateDirectoryHierarchy(ConstUnicode pathName)
  * File_DeleteDirectoryTree --
  *
  *      Deletes the specified directory tree. If filesystem errors are
- *      encountered along the way, the function will continue to delete what it
- *      can but will return FALSE.
+ *      encountered along the way, the function will continue to delete what
+ *      it can but will return FALSE.
  *
  * Results:
- *      TRUE if the entire tree was deleted or didn't exist, FALSE otherwise.
+ *      TRUE   the entire tree was deleted or didn't exist
+ *      FALSE  otherwise.
  *      
  * Side effects:
  *      Deletes the directory tree from disk.
@@ -1751,6 +1943,7 @@ File_CreateDirectoryHierarchy(ConstUnicode pathName)
 Bool
 File_DeleteDirectoryTree(ConstUnicode pathName)  // IN: directory to delete
 {
+#ifndef __MINGW32__
    int i;
    int numFiles;
    Unicode base;
@@ -1758,8 +1951,15 @@ File_DeleteDirectoryTree(ConstUnicode pathName)  // IN: directory to delete
    Unicode *fileList = NULL;
    Bool sawFileError = FALSE;
 
-   if (!File_Exists(pathName)) {
+   switch (FileAttributes(pathName, NULL)) {
+   case 0:
+      break;
+   case ENOENT:
+   case ENOTDIR:
+      /* path does not exist or is inaccessible */
       return TRUE;
+   default:
+      return FALSE;
    }
 
    /* get list of files in current directory */
@@ -1774,19 +1974,46 @@ File_DeleteDirectoryTree(ConstUnicode pathName)  // IN: directory to delete
 
    for (i = 0; i < numFiles; i++) {
       Unicode curPath;
+      struct stat statbuf;
 
       curPath = Unicode_Append(base, fileList[i]);
 
-      if (File_IsDirectory(curPath)) {
-         /* is dir, recurse */
-         if (!File_DeleteDirectoryTree(curPath)) {
-            sawFileError = TRUE;
+      if (Posix_Lstat(curPath, &statbuf) == 0) {
+         switch (statbuf.st_mode & S_IFMT) {
+         case S_IFDIR:
+            /* Directory, recurse */
+            if (!File_DeleteDirectoryTree(curPath)) {
+               sawFileError = TRUE;
+            }
+            break;
+
+#if !defined(_WIN32)
+         case S_IFLNK:
+            /* Delete symlink, not what it points to */
+            if (FileDeletion(curPath, FALSE) != 0) {
+               sawFileError = TRUE;
+            }
+            break;
+#endif
+
+         default:
+            if (FileDeletion(curPath, FALSE) != 0) {
+#if defined(_WIN32)
+               if (File_SetFilePermissions(curPath, S_IWUSR)) {
+                  if (FileDeletion(curPath, FALSE) != 0) {
+                     sawFileError = TRUE;
+                  }
+               } else {
+                  sawFileError = TRUE;
+               }
+#else
+               sawFileError = TRUE;
+#endif
+            }
+            break;
          }
       } else {
-         /* is file, delete */
-         if (File_Unlink(curPath) == -1) {
-            sawFileError = TRUE;
-         }
+         sawFileError = TRUE;
       }
 
       Unicode_Free(curPath);
@@ -1806,6 +2033,9 @@ File_DeleteDirectoryTree(ConstUnicode pathName)  // IN: directory to delete
    free(fileList);
 
    return !sawFileError;
+#else
+   return FALSE;
+#endif
 }
 
 
@@ -1839,8 +2069,8 @@ File_PrependToPath(const char *searchPath,   // IN
    ASSERT(searchPath);
    ASSERT(elem);
 
-   newPath = Str_SafeAsprintf(NULL, "%s" FILE_SEARCHPATHTOKEN "%s",
-                          elem, searchPath);
+   newPath = Str_SafeAsprintf(NULL, "%s%s%s", elem, FILE_SEARCHPATHTOKEN,
+                              searchPath);
 
    n = strlen(elem);
    path = newPath + n + 1;
@@ -1890,39 +2120,44 @@ File_FindFileInSearchPath(const char *fileIn,       // IN
                           const char *cwd,          // IN
                           char **result)            // OUT
 {
-   Bool found = FALSE;
-   char *cur = NULL;
+#ifndef __MINGW32__
+   char *cur;
+   char *tok;
+   Bool found;
+   char *saveptr = NULL;
    char *sp = NULL;
    char *file = NULL;
-   char *tok;
 
    ASSERT(fileIn);
    ASSERT(cwd);
    ASSERT(searchPath);
 
    /*
-    * First check the usual places, the fullpath, and the cwd.
+    * First check the usual places - the fullpath or the cwd.
     */
 
    if (File_IsFullPath(fileIn)) {
       cur = Util_SafeStrdup(fileIn);
    } else {
-      cur = Str_SafeAsprintf(NULL, "%s"DIRSEPS"%s", cwd, fileIn);
+      cur = Str_SafeAsprintf(NULL, "%s%s%s", cwd, DIRSEPS, fileIn);
    }
 
-   if (File_Exists(cur)) {
-      goto found;
+   if (FileAttributes(cur, NULL) == 0) {
+      goto done;
    }
+
    free(cur);
+   cur = NULL;
 
    /*
     * Didn't find it in the usual places so strip it to its bare minimum and
     * start searching.
     */
+
    File_GetPathName(fileIn, NULL, &file);
 
    sp = Util_SafeStrdup(searchPath);
-   tok = strtok(sp, FILE_SEARCHPATHTOKEN);
+   tok = strtok_r(sp, FILE_SEARCHPATHTOKEN, &saveptr);
 
    while (tok) {
       if (File_IsFullPath(tok)) {
@@ -1932,37 +2167,47 @@ File_FindFileInSearchPath(const char *fileIn,       // IN
          /* Relative Path.  Prepend the cwd. */
          if (Str_Strcasecmp(tok, ".") == 0) {
             /* Don't append "." */
-            cur = Str_SafeAsprintf(NULL, "%s"DIRSEPS"%s", cwd, file);
+            cur = Str_SafeAsprintf(NULL, "%s%s%s", cwd, DIRSEPS, file);
          } else {
-            cur = Str_SafeAsprintf(NULL, "%s"DIRSEPS"%s"DIRSEPS"%s", cwd,
-                                   tok, file);
+            cur = Str_SafeAsprintf(NULL, "%s%s%s%s%s", cwd, DIRSEPS, tok,
+                                   DIRSEPS, file);
          }
       }
 
-      if (File_Exists(cur)) {
-         goto found;
+      if (FileAttributes(cur, NULL) == 0) {
+         break;
       }
+
       free(cur);
-      tok = strtok(NULL, FILE_SEARCHPATHTOKEN);
+      cur = NULL;
+
+      tok = strtok_r(NULL, FILE_SEARCHPATHTOKEN, &saveptr);
+   }
+
+done:
+   if (cur) {
+      found = TRUE;
+
+      if (result) {
+         *result = File_FullPath(cur);
+
+         if (*result == NULL) {
+            found = FALSE;
+         }
+      }
+
+      free(cur);
+   } else {
+      found = FALSE;
    }
 
-  exit:
    free(sp);
    free(file);
+
    return found;
-
-  found:
-   ASSERT(cur);
-   found = TRUE;
-   if (result) {
-      *result = File_FullPath(cur);
-
-      if (*result == NULL) {
-         found = FALSE;
-      }
-   }
-   free(cur);
-   goto exit;
+#else
+   return FALSE;
+#endif
 }
 
 
@@ -2087,17 +2332,58 @@ File_ExpandAndCheckDir(const char *dirName)
 
    if (dirName != NULL) {
       edirName = Util_ExpandString(dirName);
-      if (edirName != NULL) {
-	 if (File_IsWritableDir(edirName) == TRUE) {
-            if (edirName[strlen(edirName) - 1] == DIRSEPC) {
-               edirName[strlen(edirName) - 1] = '\0';
-            }
-	    return edirName;
-	 }
-	 free(edirName);
+
+      if ((edirName != NULL) && FileIsWritableDir(edirName)) {
+         size_t len = strlen(edirName) - 1;
+
+         if (edirName[len] == DIRSEPC) {
+            edirName[len] = '\0';
+         }
+
+	 return edirName;
       }
    }
+
    return NULL;
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * FileSleeper
+ *
+ *	Sleep for the specified number of milliseconds plus some "slop time".
+ *      The "slop time" is added to provide some "jitter" on retries such
+ *      that two or more threads don't easily get into resonance.
+ *
+ * Results:
+ *      Sonambulistic behavior
+ *
+ * Side effects:
+ *	None
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+FileSleeper(uint32 msecSleepTime)  // IN:
+{
+   static uint32 rng = 0;
+
+   rng = 1103515245*rng + 12345;  // simple, noisy RNG
+
+   /* Add some "slop" to the sleep time */
+   if (msecSleepTime < 50) {
+      msecSleepTime += rng & 1;
+   } else {
+      msecSleepTime += rng & 3;
+   }
+
+#if defined(_WIN32)
+   Sleep(msecSleepTime);
+#else
+   usleep(1000 * msecSleepTime);
+#endif
+}
 #endif // N_PLAT_NLM
