@@ -29,8 +29,16 @@
  */
 
 
+#ifndef __MINGW32__
 #include <arpa/inet.h>
+#else
+#include <windows.h>
+#endif
+
 #include <glib.h>
+#include <glib/gstdio.h>
+
+
 #ifdef VIEW_GTK
 #include <gtk/gtkalignment.h>
 #include <gtk/gtkentry.h>
@@ -40,32 +48,58 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkstock.h>
 #endif
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <libxml/uri.h>
+#ifndef __MINGW32__
 #include <netdb.h>
 #include <netinet/in.h>
 #include <stdlib.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <vector>
 
 #if defined(__linux__)
 #include <linux/if_ether.h>
 #include <net/if.h>
 #endif
-
-#if defined(__APPLE__)
-#include <mach-o/dyld.h>
+#else // __MINGW32__
+#include <winsock2.h>
+#include <libintl.h>
+#include <unistd.h>
+#include <ws2tcpip.h>
 #endif
 
+#if defined(__APPLE__)
+#include <sys/types.h>
+/*
+ * From getifaddrs(3):
+ *
+ * If both <net/if.h> and <ifaddrs.h> are being included, <net/if.h>
+ * must be included before <ifaddrs.h>.
+ */
+#include <net/if.h>
+#include <net/if_dl.h>
+#include <ifaddrs.h>
+#include <mach-o/dyld.h>
+#include <sys/sysctl.h>
+#endif
+
+#ifdef HAVE_UIDNA_IDNTOASCII
 extern "C" {
 #include <unicode/uidna.h>
 }
+#endif // HAVE_UIDNA_IDNTOASCII
 
 
 #include "util.hh"
 #ifdef VIEW_GTK
 #include "app.hh"
 #endif
+#ifdef __APPLE__
+#include "cdkProxy.h"
+#endif
+#include "cdkUrl.h"
+
 
 extern "C" {
 #include "file.h"
@@ -74,7 +108,7 @@ extern "C" {
 }
 
 
-#ifdef VIEW_GTK
+#ifdef GDK_WINDOWING_X11
 // Here to avoid conflict with vm_basic_types.h
 #include <gdk/gdkx.h>
 #endif
@@ -334,8 +368,7 @@ CreateButton(const string stockId, // IN
  * cdk::Util::SetButtonIcon --
  *
  *      Changes the image and label displayed in the button.  This button must
- *      have been created by the Util::CreateButton method.  gtk_widget_show
- *      is also called on the button before the function returns.
+ *      have been created by the Util::CreateButton method.
  *
  * Results:
  *      None
@@ -366,8 +399,6 @@ SetButtonIcon(GtkButton *button,     // IN
    }
 
    gtk_label_set_text_with_mnemonic(l, label.c_str());
-
-   gtk_widget_show(GTK_WIDGET(button));
 }
 
 
@@ -429,6 +460,7 @@ CreateActionArea(GtkButton *button1, // IN
  *-----------------------------------------------------------------------------
  */
 
+#ifdef GDK_WINDOWING_X11
 void
 OverrideWindowUserTime(GtkWindow *window) // IN
 {
@@ -459,6 +491,7 @@ OverrideWindowUserTime(GtkWindow *window) // IN
                        GDK_PROP_MODE_REPLACE, (guchar*)&evTime, 1);
 #endif
 }
+#endif
 
 
 #endif // VIEW_GTK
@@ -494,94 +527,6 @@ UserWarning(const char *format, // IN
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * cdk::Util::IDNToASCII --
- *
- *      Convert a string from UTF-8/IDN to Punycode/ASCII.
- *
- *      For explanations of IDN, see:
- *      https://developer.mozilla.org/En/Internationalized_Domain_Names_(IDN)_Support_in_Mozilla_Browsers
- *      http://www.ietf.org/rfc/rfc3490.txt
- *      http://www.ietf.org/rfc/rfc3491.txt
- *      http://www.ietf.org/rfc/rfc3492.txt
- *
- * Results:
- *      An ASCII representation of the UTF-8 hostname, or an empty
- *      string if the conversion failed.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-string
-IDNToASCII(const string &text) // IN
-{
-   /*
-    * To convert UTF-8 to ASCII, first we need to convert it to UTF-16
-    * for ICU, then convert it to 16-bit "ASCII", and then back to
-    * 8-bit ASCII (which is ASCII) for xmlParse().
-    */
-   long utf16Len;
-   GError *error = NULL;
-   gunichar2 *utf16Text = g_utf8_to_utf16(text.c_str(), -1, NULL, &utf16Len,
-                                          &error);
-   if (error) {
-      Log("Could not convert text \"%s\" to UTF-16: %s\n", text.c_str(),
-          error->message);
-      g_error_free(error);
-      return "";
-   }
-
-   int32_t idnLen = 2 * utf16Len;
-   UChar *idnText;
-   UErrorCode status;
-   int32_t len;
-   /*
-    * If we don't allocate enough characters on the first attempt,
-    * IDNToASCII() will return how many characters we need.  If the
-    * second attempt fails, just give up rather than looping forever.
-    */
-tryConversion:
-   idnText = g_new(UChar, idnLen + 1);
-   status = U_ZERO_ERROR;
-   len = uidna_IDNToASCII((const UChar *)utf16Text, utf16Len,
-                          idnText, idnLen, UIDNA_DEFAULT, NULL,
-                          &status);
-   if (len > idnLen) {
-      g_free(idnText);
-      // Guard against multiple loops.
-      if (idnLen != 2 * utf16Len) {
-         Log("The ASCII length was greater than we allocated after two "
-             "attempts; giving up on \"%s\".", text.c_str());
-         g_free(idnText);
-         g_free(utf16Text);
-         return "";
-      }
-      idnLen = len;
-      goto tryConversion;
-   } else if (U_FAILURE(status)) {
-      Log("Could not convert text \"%s\" to IDN: %s\n", text.c_str(),
-          u_errorName(status));
-      g_free(idnText);
-      g_free(utf16Text);
-      return "";
-   }
-
-   // Convert it back to UTF-8/ASCII.
-   char *utf8Idn = g_utf16_to_utf8((gunichar2 *)idnText, len, NULL, NULL,
-                                   NULL);
-   string ret(utf8Idn);
-   g_free(idnText);
-   g_free(utf8Idn);
-   g_free(utf16Text);
-   return ret;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -640,51 +585,17 @@ ParseHostLabel(const string &label,  // IN
 	       unsigned short *port, // OUT/OPT
 	       bool *secure)         // OUT/OPT
 {
-   string text(label);
-   /*
-    * If there are multibyte characters, we need to convert from IDN
-    * to ASCII (punycode).
-    */
-   if (g_utf8_strlen(text.c_str(), -1) != (long)text.length()) {
-      text = IDNToASCII(text);
-   }
-   if (text.empty()) {
-      return text;
-   }
-
-   xmlURIPtr parsed = xmlParseURI(text.c_str());
-   if (parsed == NULL || parsed->server == NULL) {
-      text = "https://" + text;
-      /* xmlParseURI requires that the protocol be specified in order to parse
-       * correctly.  In the event that the parsing fails, this line will add
-       * the protocol in as "https://" and attempt to reparse.  If it fails
-       * the second time, it will fall back to default values.
-       */
-      parsed = xmlParseURI(text.c_str());
-   }
-
-   /*
-    * For some invalid URLs, path will contain a bunch of stuff.
-    * Since the user shouldn't be specifying a path here, use that to
-    * catch a few strange parsing errors.
-    * See bugs 379938 & 367370.
-    */
-   if (parsed != NULL &&
-       (parsed->path == NULL || parsed->path[0] == '\0') &&
-       parsed->server && strlen(parsed->server) <= MAX_HOSTNAME_LENGTH) {
-      bool tmpSecure = !parsed->scheme || !strcmp(parsed->scheme, "https");
+   string ret;
+   gboolean aSecure = true;
+   char *host = NULL;
+   if (CdkUrl_Parse(label.c_str(), NULL, &host, port, NULL, &aSecure)) {
       if (secure) {
-	 *secure = tmpSecure;
+         *secure = aSecure;
       }
-      if (port) {
-	 *port = parsed->port ? parsed->port : (tmpSecure ? 443 : 80);
-      }
-      text = parsed->server ? parsed->server : "";
-   } else {
-      text = "";
+      ret = host;
+      g_free(host);
    }
-   xmlFreeURI(parsed);
-   return text;
+   return ret;
 }
 
 
@@ -723,6 +634,10 @@ GetUsefulPath(const string systemPath,   // IN
    if (rv != 0) {
       ASSERT(rv == 0);
    }
+#elif defined(__MINGW32__)
+   self = (char *)malloc(FILE_MAXPATH);
+   ASSERT(self);
+   GetModuleFileName(NULL, self,  FILE_MAXPATH);
 #else
    self = Posix_ReadLink("/proc/self/exe");
    ASSERT(self);
@@ -736,9 +651,12 @@ GetUsefulPath(const string systemPath,   // IN
    Util::string selfPath(dirname);
    free(dirname);
 
+#ifdef BINDIR
    if (selfPath == BINDIR) {
       selfPath = systemPath;
-   } else {
+   } else
+#endif
+   {
       selfPath += G_DIR_SEPARATOR_S;
       selfPath += relativePath;
    }
@@ -750,6 +668,35 @@ GetUsefulPath(const string systemPath,   // IN
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util::GetClientHostName --
+ *
+ *      Attempts to determine the hostname for this machine.
+ *
+ * Results:
+ *      The hostname of this machine, or an empty string.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+string
+GetClientHostName()
+{
+   char name[MAX_HOSTNAME_LENGTH + 1];
+   if (gethostname(name, MAX_HOSTNAME_LENGTH) == 0) {
+      name[MAX_HOSTNAME_LENGTH] = '\0';
+      return name;
+   } else {
+      Warning("gethostname() failed: %s\n", strerror(errno));
+      return "";
+   }
+}
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -765,19 +712,18 @@ GetUsefulPath(const string systemPath,   // IN
  *-----------------------------------------------------------------------------
  */
 
+#if defined(__linux__) || defined(__APPLE__)
 ClientInfoMap
 GetClientInfo(const string broker, // IN
               int port)            // IN
 {
    ClientInfoMap info = GetNICInfo(broker, port);
 
-   char name[MAX_HOSTNAME_LENGTH + 1];
-   if (gethostname(name, MAX_HOSTNAME_LENGTH) == 0) {
-      name[MAX_HOSTNAME_LENGTH] = '\0';
-      info["Machine_Name"] = name;
-   } else {
-      Warning("gethostname() failed: %s\n", strerror(errno));
+   string hostname = GetClientHostName();
+   if (!hostname.empty()) {
+      info["Machine_Name"] = hostname;
    }
+   char name[MAX_HOSTNAME_LENGTH + 1];
    if (getdomainname(name, MAX_HOSTNAME_LENGTH) == 0) {
       name[MAX_HOSTNAME_LENGTH] = '\0';
       info["Machine_Domain"] = name;
@@ -794,9 +740,22 @@ GetClientInfo(const string broker, // IN
       info["Language"] = lang;
    }
 
-   time_t now = time(NULL);
-   struct tm tm_now;
-   localtime_r(&now, &tm_now);
+#ifdef __APPLE__
+   info["Type"] = "Mac";
+
+   /*
+    * In Obj-C++ you can simply do [[NZimeZone localTimeZone] name].
+    */
+   CFTimeZoneRef zone = CFTimeZoneCopyDefault();
+   CFStringRef zoneName = CFTimeZoneGetName(zone);
+   char *tzid = CdkProxy_CFStringToUTF8CString(zoneName);
+   if (tzid) {
+      info["TZID"] = tzid;
+      free(tzid);
+   }
+   CFRelease(zone);
+#else // !__APPLE__
+   info["Type"] = "Linux";
 
    const char *tzid = getenv("TZ");
    char *contents = NULL;
@@ -829,7 +788,7 @@ GetClientInfo(const string broker, // IN
             "exit 0\n"
          };
 
-         char *bash[] = { "/bin/bash", NULL };
+         char *bash[] = { (char *)"/bin/bash", NULL };
          GPid pid = -1;
          int std_in = -1;
          int std_out = -1;
@@ -852,7 +811,7 @@ GetClientInfo(const string broker, // IN
                 // Read time zone result from bash script.
                 if (read(std_out, contents, maxTzIdLen - 1) != -1) {
                     if (contents[0] != '\0') {
-                        /* 
+                        /*
                          * bash script returns timezone gleaned from clock
                          * file: strip any trailing space or newline.
                          */
@@ -869,16 +828,250 @@ GetClientInfo(const string broker, // IN
          }
       }
    }
-   info["TZID"] = tzid ? tzid : tm_now.tm_zone;
+   if (tzid) {
+      info["TZID"] = tzid;
+   } else {
+      time_t now = time(NULL);
+      struct tm tm_now;
+      localtime_r(&now, &tm_now);
+      info["TZID"] = tm_now.tm_zone;
+   }
    g_free(contents);
+#endif // !__APPLE__
 
-#ifdef __APPLE__
-   info["Type"] = "Mac";
-#elif defined(__linux__)
-   info["Type"] = "Linux";
-#endif
    return info;
 }
+#endif
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util::GetClientInfo --
+ *      Collects information about the client and stores it in a std::map.
+ *
+ * Results:
+ *      A mapping of the client information.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+#ifdef _WIN32
+ClientInfoMap
+GetClientInfo(const string broker, // IN
+              int port)            // IN
+
+{
+   ClientInfoMap info = GetNICInfo(broker, port);
+
+   string hostname = GetClientHostName();
+   if (!hostname.empty()) {
+      info["Machine_Name"] = hostname;
+   }
+
+#if 0
+   // XXX - this is incomplete
+   char name[MAX_HOSTNAME_LENGTH + 1];
+   if (getdomainname(name, MAX_HOSTNAME_LENGTH) == 0) {
+      name[MAX_HOSTNAME_LENGTH] = '\0';
+      info["Machine_Domain"] = name;
+   } else {
+      Warning("getdomainname() failed: %s\n", strerror(errno));
+   }
+#endif
+
+   info["LoggedOn_Username"] = g_get_user_name();
+
+#if 0
+   // XXX - incomplete - no setlocale in Windows.
+   string lang = setlocale(LC_MESSAGES, NULL);
+#endif
+   string lang = "C";
+   if (lang == "C" || lang == "POSIX" || lang == "") {
+      info["Language"] = "en";
+   } else {
+      info["Language"] = lang;
+   }
+
+   TIME_ZONE_INFORMATION tzInfo;
+   char tzid[(sizeof(tzInfo.StandardName) / sizeof(wchar_t)) + 1];
+   wchar_t *tzname = NULL;
+   int tztype = GetTimeZoneInformation(&tzInfo);
+
+   if (tztype == TIME_ZONE_ID_STANDARD) {
+      tzname = tzInfo.StandardName;
+   } else if (tztype == TIME_ZONE_ID_DAYLIGHT) {
+      tzname = tzInfo.DaylightName;
+   }
+   if (tzname && wcstombs(tzid, tzname, sizeof(tzid)) > 0) {
+      // wcstobms does not guarantee null termination of tzid so just to be sure...
+      tzid[sizeof(tzid)-1] = '\0';
+      info["Windows_Timezone"] = g_strdup(tzid);
+   } else {
+      Warning("Unable to determine time zone.");
+   }
+
+   info["Type"] = "Windows";
+
+   return info;
+}
+#endif
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util::GetMacAddr --
+ *
+ *      Linux implementation of GetMacAddr.
+ *
+ * Results:
+ *      The mac address of the given IPv4 address.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+#ifdef __linux__
+string
+GetMacAddr(int sock,                 // IN
+           struct sockaddr_in *addr) // IN
+{
+   struct ifreq ifr;
+   int i = 0;
+   while (++i) {
+      ifr.ifr_ifindex = i;
+      /*
+       * First, fetch the device's name. If the device does not exist,
+       * ioctl() will return -1 and the loop will end.
+       */
+      if (ioctl(sock, SIOCGIFNAME, &ifr) < 0 ) {
+         break;
+      }
+      /*
+       * Then, fetch the device's IP address.  If it does not have an IP,
+       * go on to the next device.
+       */
+      if (ioctl(sock, SIOCGIFADDR, &ifr) < 0) {
+         continue;
+      }
+
+      /*
+       * Compare this IP address to the one obtained from getsockname().
+       * If they match, then we use ioctl again to obtain the MAC
+       * address of that device.
+       */
+      if (ifr.ifr_addr.sa_family != addr->sin_family) {
+         continue;
+      }
+      struct sockaddr_in *ifraddr = (struct sockaddr_in *)&ifr.ifr_addr;
+      if (!memcmp(&addr->sin_addr, &ifraddr->sin_addr,
+                  sizeof(addr->sin_addr))) {
+         // Get the MAC Address of the network card.
+         if (ioctl(sock, SIOCGIFHWADDR, &ifr) < 0) {
+             Warning("ioctl() failed to get the local MAC address while "
+                     "collecting client info: %s\n", strerror(errno));
+             break;
+         }
+         return Format("%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
+                       (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[0],
+                       (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[1],
+                       (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[2],
+                       (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[3],
+                       (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[4],
+                       (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[5]);
+      }
+   }
+   return "";
+}
+#endif // __linux__
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util::GetMacAddr --
+ *
+ *      Mac/BSD Implementation of GetMacAddr.
+ *
+ * Results:
+ *      The Mac address for the given IPv4 address.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+#ifdef __APPLE__
+string
+GetMacAddr(int sock,                 // IN
+           struct sockaddr_in *addr) // IN
+{
+   string ret;
+   struct ifaddrs *addrs = NULL;
+   if (getifaddrs(&addrs)) {
+      Warning("GetMacAddr: getifaddrs failed: %s\n", strerror(errno));
+      return ret;
+   }
+
+   for (struct ifaddrs *i = addrs; i; i = i->ifa_next) {
+      // Only handle IPv4 addresses.
+      if (!i->ifa_addr || i->ifa_addr->sa_family != addr->sin_family) {
+         continue;
+      }
+
+      // This interface has the same address as our socket.
+      struct sockaddr_in *i_addr = (struct sockaddr_in *)i->ifa_addr;
+      if (memcmp(&i_addr->sin_addr, &addr->sin_addr, sizeof(addr->sin_addr))) {
+         continue;
+      }
+
+      /*
+       * Well, this is the interface we want; if any of these fail, we
+       * should just bail out of this loop anyway.
+       */
+      int mib[6] = { CTL_NET, AF_ROUTE, 0, AF_LINK, NET_RT_IFLIST };
+      size_t len;
+      mib[5] = if_nametoindex(i->ifa_name);
+      if (mib[5] == 0) {
+         Warning("GetMacAddr: if_nametoindex failed: %s\n", strerror(errno));
+         break;
+      }
+      if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0) {
+         Warning("GetMacAddr: sysctl 1 failed: %s\n", strerror(errno));
+         break;
+      }
+      char *buf = (char *)g_malloc(len);
+      if (sysctl(mib, 6, buf, &len, NULL, 0) < 0) {
+         Warning("GetMacAddr: sysctl 2 failed: %s\n", strerror(errno));
+         break;
+      }
+
+      struct if_msghdr *ifm = (struct if_msghdr *)buf;
+      struct sockaddr_dl *sdl = (struct sockaddr_dl *)(ifm + 1);
+      unsigned char *ptr = (unsigned char *)LLADDR(sdl);
+
+      ret = Format("%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
+                   (unsigned int)(unsigned char)ptr[0],
+                   (unsigned int)(unsigned char)ptr[1],
+                   (unsigned int)(unsigned char)ptr[2],
+                   (unsigned int)(unsigned char)ptr[3],
+                   (unsigned int)(unsigned char)ptr[4],
+                   (unsigned int)(unsigned char)ptr[5]);
+
+      g_free(buf);
+      break;
+   }
+   freeifaddrs(addrs);
+   return ret;
+}
+#endif // __APPLE__
 
 
 /*
@@ -898,6 +1091,7 @@ GetClientInfo(const string broker, // IN
  *-----------------------------------------------------------------------------
  */
 
+#if defined(__linux__) || defined(__APPLE__)
 ClientInfoMap
 GetNICInfo(const string broker, // IN
            int port)            // IN
@@ -911,16 +1105,27 @@ GetNICInfo(const string broker, // IN
       return info;
    }
 
-   int sock = socket(AF_INET, SOCK_STREAM, 0);
+   if (serverInfo->h_addrtype != AF_INET) {
+      Warning("Skipping NIC info for non-IPv4 address of type %d\n",
+              serverInfo->h_addrtype);
+      return info;
+   }
+
+   struct sockaddr_in server;
+   if ((size_t)serverInfo->h_length > sizeof(server.sin_addr.s_addr)) {
+      Warning("Server address is larger than expected socket address\n");
+      return info;
+   }
+
+   int sock = socket(serverInfo->h_addrtype, SOCK_DGRAM, 0);
    if (sock < 0) {
       Warning("socket() failed while compiling client info: %s\n",
               strerror(errno));
       return info;
    }
 
-   struct sockaddr_in server;
    memcpy(&server.sin_addr.s_addr, serverInfo->h_addr, serverInfo->h_length);
-   server.sin_family = AF_INET;
+   server.sin_family = serverInfo->h_addrtype;
    server.sin_port = htons(port);
 
    if (connect(sock, (struct sockaddr *)&server, sizeof(server)) < 0) {
@@ -930,71 +1135,192 @@ GetNICInfo(const string broker, // IN
       return info;
    }
 
-   struct sockaddr local;
-   socklen_t addrSize = sizeof(local);
-   if (getsockname(sock, &local, &addrSize) < 0) {
-      Warning("getsockname() failed to get the local IP address while "
-              "compiling client info: %s\n", strerror(errno));
-      close(sock);
-      return info;
+   struct sockaddr_in local;
+   {
+      struct sockaddr local_sa;
+      socklen_t addrSize = sizeof(local_sa);
+      if (getsockname(sock, &local_sa, &addrSize) < 0) {
+         Warning("getsockname() failed to get the local IP address while "
+                 "compiling client info: %s\n", strerror(errno));
+         close(sock);
+         return info;
+      }
+      memcpy(&local, &local_sa, sizeof(local));
    }
 
-   char buffer[256];
-   if (inet_ntop(local.sa_family, &local.sa_data[2], buffer,
-                 sizeof(buffer)) == NULL) {
-      Warning("inet_ntop() failed to format the local IP address while "
-              "compiling client info: %s\n", strerror(errno));
-      close(sock);
-      return info;
+   {
+      char buffer[256];
+      if (inet_ntop(local.sin_family, &local.sin_addr, buffer,
+                    sizeof(buffer)) == NULL) {
+         Warning("inet_ntop() failed to format the local IP address while "
+                 "compiling client info: %s\n", strerror(errno));
+         close(sock);
+         return info;
+      }
+      info["IP_Address"] = buffer;
    }
-   info["IP_Address"] = buffer;
 
-#ifdef __linux__
-   struct ifreq ifr;
-   int i = 0;
-   while (++i) {
-      ifr.ifr_ifindex = i;
-      /*
-       * First, fetch the device's name. If the device does not exist,
-       * ioctl() will return -1 and the loop will end.
-       */
-      if (ioctl(sock, SIOCGIFNAME, &ifr) < 0 ) {
-         break;
-      }
-      /*
-       * Then, fetch the device's IP address.  If it does not have an IP,
-       * go on to the next device.
-       */
-      if (ioctl(sock, SIOCGIFADDR, &ifr) < 0) {
-         continue;
-      }
-      /*
-       * Compare this IP address to the one obtained from getsockname().
-       * If they match, then we use ioctl again to obtain the MAC
-       * address of that device.
-       */
-      if (memcmp(&local.sa_data[2], &ifr.ifr_addr.sa_data[2],
-          4 * sizeof(local.sa_data[2])) == 0) {
-         // Get the MAC Address of the network card.
-         if (ioctl(sock, SIOCGIFHWADDR, &ifr) < 0) {
-             Warning("ioctl() failed to get the local MAC address while "
-                     "compiling client info: %s\n", strerror(errno));
-         } else {
-            info["MAC_Address"] = Format("%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
-                      (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[0],
-                      (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[1],
-                      (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[2],
-                      (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[3],
-                      (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[4],
-                      (unsigned int)(unsigned char)ifr.ifr_hwaddr.sa_data[5]);
-         }
-         break;
-      }
-   }
-#endif
+   info["MAC_Address"] = GetMacAddr(sock, &local);
+
    close(sock);
    return info;
 }
+#endif
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util::GetNICInfo --
+ *      Gets the IP and MAC address of the active network card and adds them
+ *      to info.
+ *
+ * Results:
+ *      If the IP and MAC addresses can be determined, they will be added to
+ *      info.  If they can not, info will be left alone.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+#ifdef _WIN32
+ClientInfoMap
+GetNICInfo(const string broker, // IN
+           int port)            // IN
+{
+   // XXX - need windows implementation.
+   ClientInfoMap info;
+   return info;
+}
+#endif
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util:Utf8Casecmp: --
+ *
+ *      Compare two UTF-8 strings, ignoring case.
+ *
+ * Results:
+ *      An int less than, greater than, or equal to zero if s1 is less
+ *      than, greater than, or equal to s2.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+Utf8Casecmp(const char *s1, // IN
+            const char *s2) // IN
+{
+   ASSERT(s1);
+   ASSERT(s2);
+
+   char *folded1 = g_utf8_casefold(s1, -1);
+   char *folded2 = g_utf8_casefold(s2, -1);
+
+   int ret = g_utf8_collate(folded1, folded2);
+
+   g_free(folded1);
+   g_free(folded2);
+
+   return ret;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util:MkdirWithParents: --
+ *
+ *      Create a directory and, if needed, its parent directories
+ *
+ * Results:
+ *      0 if the directory was created successfully.
+ *      -1 if an error occurred, with errno set by g_mkdir.
+ *
+ * Side effects:
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+MkdirWithParents(const char *path, // IN
+                 int mode)         // IN
+{
+#if GLIB_CHECK_VERSION(2, 8, 0)
+   return g_mkdir_with_parents(path, mode);
+#else
+   char *dirPath = g_str_has_suffix(path, G_DIR_SEPARATOR_S)
+                      ? g_strdup(path)
+                      : g_strdup_printf("%s%c", path, G_DIR_SEPARATOR);
+   bool skipFirst = g_path_is_absolute(path);
+   for (char *c = dirPath; *c != '\0'; c++) {
+      if (*c != G_DIR_SEPARATOR) {
+         continue;
+      } else if (skipFirst) {
+         skipFirst = false;
+         continue;
+      }
+      *c = '\0';
+      if (g_mkdir(dirPath, mode) && errno != EEXIST) {
+         int errnoSave = errno;
+         g_free(dirPath);
+         errno = errnoSave;
+         return -1;
+      }
+      *c = G_DIR_SEPARATOR;
+   }
+   g_free(dirPath);
+   return 0;
+#endif
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * cdk::Util::EnsureFilePermissions --
+ *
+ *      Create the file path with the given mode, or ensure the
+ *      existing file has the given mode.
+ *
+ * Results:
+ *      true if file was OK.
+ *
+ * Side effects:
+ *      Creates path.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+bool
+EnsureFilePermissions(const char *path, // IN
+                      int mode)         // IN
+{
+   bool fileOk = false;
+   if (0 == chmod(path, mode)) {
+      fileOk = true;
+   } else if (errno == ENOENT) {
+      int fd = open(path, O_CREAT, mode);
+      if (fd != -1) {
+         fileOk = true;
+         close(fd);
+      } else  {
+            Warning(_("File '%s' could not be created: %s\n"),
+                    path, strerror(errno));
+      }
+   } else {
+      Warning(_("Could not change mode of file '%s': %s\n"),
+              path, strerror(errno));
+   }
+   return fileOk;
+}
 
 
 } // namespace Util
